/***************************************************************************
                          controlpanel.h  -  description
                             -------------------
    begin                : Don Apr 17 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTROLPANEL_H
#define CONTROLPANEL_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "debug.h"

#include <kpushbutton.h>
#include <kcombobox.h>

#include <qwidget.h>
#include <qslider.h>
#include <qpixmap.h>


class ControlPanel : public QWidget  {
   Q_OBJECT
public: 
	ControlPanel(QWidget *parent=0, const char *name=0);
	~ControlPanel();

  void setVolume(int vol);
  void ResetControlPanel();
  void SetPlaying();
  bool GetPlaying() const;
  
  
signals:
  void signalPlay();
  void signalPlayNext();
  void signalPlayPrevious();
  void signalPauseOn();
  void signalPauseOff();
  void signalStop();
  void signalVolumeChanged(int);
  void signalSetAudioChannel(int);
  void signalSetSubtitleChannel(int);
  void signalChangePosition(int);
  void signalTogglePlayMode();
  void signalPlayList();

public slots:
  void slotNewChannels(const QStringList& audio, const QStringList& subtitles, int currentAudio, int currentSub);
  void slotNewPosition(int pos, const QString& time);
   
private slots:
  void slotPlayPause();
  void slotStop();
  void slotNext();
  void slotPrev();
  void slotPosPlus();
  void slotPosMinus();

private:
 KPushButton* playPausePB;

 KPushButton* nextPB;
 KPushButton* prevPB;
 KPushButton* stopPB;
 KPushButton* timeButton;
 KPushButton* playListPB;
 
 QSlider* posSlider;
 QSlider* volSlider;

 KComboBox* audioChannelCB;
 KComboBox* subtitleCB;
 
 QPixmap playPixmap;
 QPixmap pausePixmap;

 bool isPlaying;
 bool isPlayButton;
 
};

#endif
