/***************************************************************************
                          kmp.cpp  -  description
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <kglobal.h>
#include <kiconloader.h>
#include <kurlrequesterdlg.h>
#include <kurldrag.h>

#include <qapplication.h>


#include "screenshotpreview.h"
#include "kaffeine.h"


KMP::KMP(QWidget *parent, const char *name) : QWidget(parent, name), playSingleUrl(false),
                                            cUrl(QString::null), cTitle(QString::null)
{

  QString version = VERSION;
  LOG("Starting Kaffeine " << version << "...")
  LOG("kaffeine@gmx.at")
  LOG("http://members.chello.at/kaffeine")
  cerr << endl;
 
  setCaption("Kaffeine Media Player " + version);
       

  QVBoxLayout* boxLayout = new QVBoxLayout(this);
  videoWin= new VideoWindow(this, "videowin");
  cPanel = new ControlPanel(this, "controlpanel");
  tRow = new TextRow(this, "inforow");
  
  osDisplay = new SplashWidget(0, "osd");


//******* check/create ./kaffeine dir
  QDir homedir = QDir::home();
  if (!homedir.exists(".kaffeine"))
    {
      PRINT("Creating /.kaffeine directory");
      homedir.mkdir(".kaffeine");
    }  
  
  
  InitActions();
  InitMenu();
  InitConnections();

  boxLayout->addWidget(menuBar);
  boxLayout->addWidget(videoWin);
  boxLayout->addWidget(tRow);
  boxLayout->addWidget(cPanel);


  setAcceptDrops(true);

  connect(&playDelayedTimer, SIGNAL(timeout()), this, SLOT(slotPlay()));
  connect(&playLOGODelayedTimer, SIGNAL(timeout()), this, SLOT(slotPlayLOGO()));

// xine is not running:
  cPanel->blockSignals(true);

}

KMP::~KMP()
{
  delete bmenu;
}


void KMP::Finalize()
{
//******** autoplay plugins ******
  QStringList autoPlayList;
  unsigned int i=0;
  QSignalMapper* mapper = new QSignalMapper(this);
  connect(mapper, SIGNAL(mapped(const QString&)), this, SLOT(slotPlayAutoplayPlugin(const QString&)));
  
  videoWin->GetAutoplayPlugins(autoPlayList);
  while(i<autoPlayList.count())
  {
   autoPlayPlugin = new KAction(autoPlayList[i], "cdrom_unmount", 0, mapper, SLOT(map()), actionCollection);
   autoPlayPlugin->plug(playMenu);
   mapper->setMapping(autoPlayPlugin, autoPlayList[i]);
   i++;
   autoPlayPlugin->setWhatsThis(autoPlayList[i]);
   i++;
  }

  cPanel->setVolume(videoWin->GetVolume());

//****** playlist  ***
  playList = new PlayList(videoWin->GetMimeTypes(), 0, "playlist");

//******* video settings ****
  
  int hue, sat, contrast, bright, audioAmp, avOffset, spuOffset;

  videoWin->GetVideoSettings(hue, sat, contrast, bright, audioAmp, avOffset, spuOffset);
  
  videoSettings = new VideoSettings(hue, sat, contrast, bright, audioAmp, avOffset,
                                    spuOffset, 0, "videosettings");
  
  

//******* connections with playlist and videosettings ********
        
  connect(playList, SIGNAL(signalPlayDirect()), this, SLOT(slotPlay()));
  connect(videoWin, SIGNAL(signalNewMrlReference(const QString&)), playList, SLOT(slotAddMrl(const QString&)));
  connect(videoWin, SIGNAL(signalLengthInfo(const QString&)),
          playList, SLOT(slotGetLengthInfo(const QString&)));
  connect(videoWin, SIGNAL(signalMetaInfo(const QString&)), playList, SLOT(slotGetMetaInfo(const QString&)));
  connect(videoWin, SIGNAL(signalStreamInfo(const QString&)), playList, SLOT(slotGetStreamInfo(const QString&)));
  connect(videoWin, SIGNAL(signalShowOSD(const QString&)),
          osDisplay, SLOT(slotOsdShow(const QString&)));
  connect(videoSettings, SIGNAL(signalNewHue(int)), videoWin, SLOT(slotSetHue(int)));
  connect(videoSettings, SIGNAL(signalNewSaturation(int)), videoWin, SLOT(slotSetSaturation(int)));
  connect(videoSettings, SIGNAL(signalNewContrast(int)), videoWin, SLOT(slotSetContrast(int)));
  connect(videoSettings, SIGNAL(signalNewBrightness(int)), videoWin, SLOT(slotSetBrightness(int)));
  connect(videoSettings, SIGNAL(signalNewAudioAmp(int)), videoWin, SLOT(slotSetAudioAmp(int)));
  connect(videoSettings, SIGNAL(signalNewAVOffset(int)), videoWin, SLOT(slotSetAVOffset(int)));
  connect(videoSettings, SIGNAL(signalNewSpuOffset(int)), videoWin, SLOT(slotSetSpuOffset(int)));

 
  
  cPanel->blockSignals(false);
  tRow->slotChangeText(QString(i18n("Ready.")));

//******** config stuff

  config = kapp->config();
  ReadConfig();

}


void KMP::SaveConfig()
{
  QFont f;
  QColor c;
  QString mstr;
 

  PRINT("Save config")

  config->setGroup("General Options");
  config->writeEntry("KMP Geometry", size());
  config->writeEntry("PlayList Geometry", playList->size());
  config->writeEntry("PlayList visible", playList->isVisible());
  config->writeEntry("VideoSettings Geometry", videoSettings->size());
  config->writeEntry("Endless Mode", playEndlessPlay->isChecked());
  config->writeEntry("Random Mode", playRandomPlay->isChecked());

  config->writeEntry("Load Playlist", loadPlaylistAtStartup);
  config->writeEntry("Current Playlist:", playList->GetLastPlaylist());

  config->writeEntry("Meta String", videoWin->GetMetaString());
  config->writeEntry("Screensaver Timeout", videoWin->GetScreensaverTimeout());

  tRow->GetConfig(f, c);
  config->writeEntry("Textrow Font", f);
  config->writeEntry("Textrow Color", c);

  config->setGroup("OSD Options");
  bool u, a;
  QFont font;
  QColor color;
  int x, y, s ,t;
  osDisplay->GetConfig(u, a, s, font, color, t, x, y);
  config->writeEntry("Use OSD", u);
  config->writeEntry("Animate", a);
  config->writeEntry("Animation Speed",s );
  config->writeEntry("OSD Font", font);
  config->writeEntry("OSD Color", color);
  config->writeEntry("Display Time", t);
  config->writeEntry("X-Pos", x);
  config->writeEntry("y-Pos", y);

  playRecent->saveEntries(config, "Recent Files");
  
}

void KMP::ReadConfig()
{
  QSize size;
  QFont f;
  QColor c;
  bool b;

  PRINT("Read config")

  config->setGroup("General Options");
  size = QSize(450, 420);
  size = config->readSizeEntry("KMP Geometry", &size);
  resize(size);
  show();

  size = QSize(350,350);
  size = config->readSizeEntry("PlayList Geometry", &size);
  playList->resize(size);
  b = config->readBoolEntry("PlayList visible", false);
  if (b) playList->show();

  size = QSize(350,100);
  size = config->readSizeEntry("VideoSettings Geometry", &size);
  videoSettings->resize(size);

  b = config->readBoolEntry("Endless Mode", false);
  playEndlessPlay->setChecked(b);
  playList->SetEndless(b);
     
  b = config->readBoolEntry("Random Mode", false);
  playRandomPlay->setChecked(b);
  playList->SetRandomPlay(b);

  loadPlaylistAtStartup = config->readBoolEntry("Load Playlist", true);
  QString pl = config->readEntry("Current Playlist:", QString());
  playList->SetLastPlaylist(pl);
  
  int ssTimeout = config->readNumEntry("Screensaver Timeout", 2);
  QString metaStr = config->readEntry("Meta String", QString("%1 - %2 (%3)"));
  playList->slotSetMetaInfoString(metaStr);
  videoWin->slotSetConfig(metaStr, ssTimeout);

  f = KGlobalSettings::generalFont();
  QFont infoFont = config->readFontEntry("Textrow Font", &f);
  c = KGlobalSettings::baseColor();
  QColor infoColor = config->readColorEntry("Textrow Color", &c);
  tRow->slotSetConfig(&infoFont, &infoColor);

  config->setGroup("OSD Options");
  bool osdUse, osdAnim;
  QFont osdFont;
  QColor osdColor;
  int osdX, osdY, osdAnimSpeed, osdTime;
  
  osdUse = config->readBoolEntry("Use OSD", true);
  osdAnim = config->readBoolEntry("Animate", true);
  osdAnimSpeed = config->readNumEntry("Animation Speed", 15 );
  f.setPointSize(18);
  osdFont = config->readFontEntry("OSD Font", &f);
  c = QColor(0xFF, 0x89, 0);
  osdColor = config->readColorEntry("OSD Color", &c);
  osdTime = config->readNumEntry("Display Time", 2);
  osdX = config->readNumEntry("X-Pos", 5);
  osdY = config->readNumEntry("y-Pos", 5);

  playRecent->loadEntries(config, "Recent Files");

  osDisplay->slotSetConfig(osdUse, osdAnim, osdAnimSpeed,  &osdFont, &osdColor,
                           osdTime, osdX, osdY);

  configWin = new ConfigWidget(loadPlaylistAtStartup, ssTimeout, &metaStr,
                  &infoFont, &infoColor,osdUse, osdAnim, osdAnimSpeed,
                  &osdFont, &osdColor, osdTime, osdX, osdY);


 connect(configWin, SIGNAL(signalNewGenConfig(bool)), this, SLOT(slotGenConfig(bool)));
 connect(configWin, SIGNAL(signalNewInfoRowConfig(QFont*, QColor*)), tRow, SLOT(slotSetConfig(QFont*, QColor*)));
 connect(configWin, SIGNAL(signalNewVideoConfig(const QString&, int)),
                    videoWin, SLOT(slotSetConfig(const QString&, int)));
 connect(configWin, SIGNAL(signalNewOsdConfig(bool, bool, int, QFont*, QColor*, int, int, int)),
         osDisplay, SLOT(slotSetConfig(bool, bool, int, QFont*, QColor*, int, int, int)));
 connect(configWin, SIGNAL(signalMSForPlaylist(const QString&)), playList, SLOT(slotSetMetaInfoString(const QString&)));                                           
}


//******** init stuff


void KMP::InitConnections()
{
 
  connect(cPanel, SIGNAL(signalPlay()), this, SLOT(slotPlay()));
  connect(cPanel, SIGNAL(signalVolumeChanged(int)), videoWin, SLOT(slotSetVolume(int)));
  connect(videoWin, SIGNAL(signalNewInfo(const QString&)), tRow, SLOT(slotChangeText(const QString&)));
  connect(cPanel, SIGNAL(signalPlayNext()), this, SLOT(slotPlayNext()));
  connect(videoWin, SIGNAL(signalPlaybackFinished()), this, SLOT(slotEndOfPlayback()));
  connect(cPanel, SIGNAL(signalPlayPrevious()), this, SLOT(slotPlayPrev()));
  connect(cPanel, SIGNAL(signalStop()), this , SLOT(slotStop()));
  connect(videoWin, SIGNAL(signalHasChapters(bool)), this, SLOT(slotHasChapters(bool)));
  connect(cPanel, SIGNAL(signalPauseOn()), videoWin, SLOT(slotSpeedPause()));
  connect(cPanel, SIGNAL(signalPauseOff()), videoWin, SLOT(slotSpeedNormal()));
  connect(videoWin, SIGNAL(signalNewChannels(const QStringList&, const QStringList&, int, int)),
          cPanel, SLOT(slotNewChannels(const QStringList&, const QStringList&, int ,int)));
  connect(cPanel, SIGNAL(signalSetAudioChannel(int)), videoWin, SLOT(slotSetAudioChannel(int)));
  connect(cPanel, SIGNAL(signalSetSubtitleChannel(int)), videoWin, SLOT(slotSetSubtitleChannel(int)));        
  connect(videoWin, SIGNAL(signalNewPosition(int, const QString&)),
          cPanel, SLOT(slotNewPosition(int, const QString&)));
  connect(cPanel, SIGNAL(signalChangePosition(int)), videoWin, SLOT(slotChangePosition(int)));
  connect(cPanel, SIGNAL(signalTogglePlayMode()), videoWin, SLOT(slotTogglePlayMode()));
  connect(cPanel, SIGNAL(signalPlayList()), this, SLOT(slotShowPlayList()));
 
}
  

void KMP::InitActions()
{

 actionCollection= new KActionCollection(this);

 fileQuit= KStdAction::quit(this, SLOT(slotQuit()), actionCollection);
 fileOpen = new KAction(i18n("&Open File(s)"), "fileopen", Qt::CTRL|Qt::Key_O, this, SLOT(slotOpenFile()), actionCollection, "openfile");
 fileSaveScreenshot = new KAction(i18n("&SaveScreenshot"), "filesave", Qt::CTRL|Qt::Key_S, this, SLOT(slotScreenshot()), actionCollection, "screenshot");
 fileOpenUrl= new KAction(i18n("Open &URL"), "package_network", Qt::CTRL|Qt::Key_U, this, SLOT(slotOpenUrl()), actionCollection, "openurl");
 fileEject = new KAction(i18n("&Eject"), "player_eject", 0, videoWin, SLOT(slotEject()), actionCollection, "eject");
 
 playRecent = KStdAction::openRecent(this, SLOT(slotPlayRecent(const KURL&)), actionCollection);
 playEndlessPlay = new KToggleAction(i18n("&Endless Play"), 0 , Qt::CTRL|Qt::Key_E, this, SLOT(slotEndlessMode()), actionCollection,"endlessplay");
 playRandomPlay = new KToggleAction(i18n("&Random Play"), 0 , Qt::CTRL|Qt::Key_R, this, SLOT(slotRandomPlay()), actionCollection,"randomplay");
 
 viewFullscreen= new KToggleAction(i18n("&Fullscreen"), "window_fullscreen", Qt::Key_F, this, SLOT(slotToggleFullscreen()), actionCollection, "fullscreen");
 viewDeinterlace=new KToggleAction(i18n("&Deinterlace"), NULL, Qt::CTRL|Qt::Key_I, videoWin, SLOT(slotToggleDeinterlace()), actionCollection, "deinterlace");

 viewDeinterlace->setWhatsThis(i18n("Activate this for interlaced streams, some DVD's for example."));

 viewZoomIn= new KAction(i18n("Zoom &in"), "viewmag+", Qt::Key_Z, videoWin, SLOT(slotZoomIn()), actionCollection, "zoomin");
 viewZoomOut= new KAction(i18n("Zoom &out"), "viewmag-", Qt::SHIFT|Qt::Key_Z, videoWin, SLOT(slotZoomOut()), actionCollection, "zoomout");
 viewZoomOff= new KAction(i18n("Zoom o&ff"), "viewmag1", Qt::CTRL|Qt::Key_Z, videoWin, SLOT(slotZoomOff()), actionCollection, "zoomoff");
 viewToggleTv=new KToggleAction(i18n("Toggle T&V-Mode"),"tv", Qt::ALT|Qt::Key_T,this, SLOT(slotToggleTv()),actionCollection, "toggletv");

 aspectRatioAuto =new KAction(i18n("&auto"), "viewmagfit", Qt::Key_F5, videoWin, SLOT(slotAspectRatioAuto()), actionCollection, "aspectratioauto");
 aspectRatio4_3 = new KAction(i18n("&4:3"), "viewmagfit", Qt::Key_F6, videoWin, SLOT(slotAspectRatio4_3()), actionCollection, "aspectratio43");
 aspectRatio16_9 =new KAction(i18n("&16:9"), "viewmagfit", Qt::Key_F7, videoWin, SLOT(slotAspectRatio16_9()), actionCollection, "aspectratio169");
 aspectRatioSquare = new KAction(i18n("&1:1"), "viewmagfit", Qt::Key_F8, videoWin, SLOT(slotAspectRatioSquare()), actionCollection, "aspectratio12");
 
 configVideoControl= new KAction(i18n("&Video Settings"), "configure", Qt::Key_V, this, SLOT(slotVideoSettings()), actionCollection, "videocontrol");
 configXine= new KAction(i18n("Configure &Xine"), "edit", 0, this, SLOT(slotConfigXine()), actionCollection, "configxine");
 configKMP = new KAction(i18n("&Kaffeine Setup"), "kaffeine", 0, this, SLOT(slotShowSetup()), actionCollection, "setup");

 viewToggleTv->setEnabled(false);
// fileSaveScreenshot->setEnabled(false);
} 


void KMP::InitMenu()
{

 KAboutApplication* kabout= new KAboutApplication(this);
 KHelpMenu* khelp= new KHelpMenu(this);
 connect(khelp, SIGNAL(showAboutApplication()), kabout, SLOT(show()));

 menuBar= new KMenuBar(this);
 menuBar->setSizePolicy ( QSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Fixed) );

 //init bookmark menu
 bookmarkMenu = new KPopupMenu(this);
 bookmarkManager = KBookmarkManager::managerForFile(QDir::homeDirPath() + "/.kaffeine/bookmarks.xml");
 bmenu = new KBookmarkMenu(bookmarkManager, this, bookmarkMenu, actionCollection, true);

 fileMenu= new KPopupMenu(this);
 playMenu= new KPopupMenu(this);
 viewMenu= new KPopupMenu(this);
 aspectMenu = new KPopupMenu(this);
 configMenu= new KPopupMenu(this);
 helpMenu= khelp->menu();

 fileOpen->plug(fileMenu);
 fileOpenUrl->plug(fileMenu);
 fileSaveScreenshot->plug(fileMenu);
 fileMenu->insertSeparator();
 fileEject->plug(fileMenu);
 fileMenu->insertSeparator();
 fileQuit->plug(fileMenu);

 playRecent->plug(playMenu);
 playMenu->insertSeparator();
 playEndlessPlay->plug(playMenu);
 playRandomPlay->plug(playMenu);
 playMenu->insertSeparator();

 aspectRatioAuto->plug(aspectMenu);
 aspectRatio4_3->plug(aspectMenu);
 aspectRatio16_9->plug(aspectMenu);
 aspectRatioSquare->plug(aspectMenu);

 viewFullscreen->plug(viewMenu);
 viewDeinterlace->plug(viewMenu);
 viewMenu->insertSeparator();
 viewMenu->insertItem(i18n("&Aspect Ratio"), aspectMenu);
 viewZoomIn->plug(viewMenu);
 viewZoomOut->plug(viewMenu);
 viewZoomOff->plug(viewMenu);
// viewMenu->insertSeparator();
// viewToggleTv->plug(viewMenu);

 configVideoControl->plug(configMenu);
 configXine->plug(configMenu);
 configMenu->insertSeparator();
 configKMP->plug(configMenu);

 menuBar->insertItem(i18n("&File"), fileMenu);
 menuBar->insertItem(i18n("&Play"), playMenu);
 menuBar->insertItem(i18n("&View"), viewMenu);
 menuBar->insertItem(i18n("&Config"), configMenu);
 menuBar->insertItem(i18n("&Bookmarks"), bookmarkMenu);
 menuBar->insertItem(i18n("&Help"), helpMenu);
}


void KMP::LoadPlaylist()
{
  if (loadPlaylistAtStartup)
  {
    PRINT("Load Playlist")
    tRow->slotChangeText(i18n("Load Playlist..."));
    playList->ReloadLastPlaylist();
    tRow->slotChangeText(i18n("Ready."));
 }
}


void KMP::CmdArgs(const KURL::List& files, bool playAtStartup, bool startFullscreen)
{

  if (startFullscreen)
  {
    viewFullscreen->setChecked(true);
    slotToggleFullscreen();
  }

  if (files.count() > 0)
  {
    playList->Add(files, NULL);
    playDelayedTimer.start(500, true);
  }
  else
  {
   if (playAtStartup)
     playDelayedTimer.start(500, true);
    else
     playLOGODelayedTimer.start(500, true);  // else start logo
  }  

}         
 


void KMP::dragEnterEvent (QDragEnterEvent *dev)
{
  dev->accept(QUriDrag::canDecode(dev) || QTextDrag::canDecode(dev));
}


void KMP::dropEvent(QDropEvent* dev)
{
 // PRINT("Drop event, type: " << dev->format())

  KURL::List uris;
  
  if (KURLDrag::decode(dev, uris))
  {
    playList->slotClearList();
    playList->Add(uris, NULL);
    slotPlay();
  }
  else 
  if (strcmp(dev->format(), "text/x-moz-url") == 0)    // for mozilla drops
  {
    playList->slotClearList();
    QByteArray data = dev->encodedData("text/plain");
    QString md(data);
    playList->Add(md, NULL);
    slotPlay();
  }  

}  


void KMP::closeEvent(QCloseEvent* cev)
{
 cev->accept();
 slotQuit();
  
}

//*** send expose event to xine
void KMP::moveEvent(QMoveEvent*)
{ 
  videoWin->ReDraw();
} 


//******************** playback control *****************

void KMP::slotPlay()
{
  
  if (playSingleUrl) playSingleUrl = false;
  
  QListViewItem* item = playList->GetCurrent();

  if (!item) return;

  if (videoWin->PlayMRL(item->text(4), item->text(1), true))
  {
    cPanel->SetPlaying();
    playRecent->addURL(item->text(4));
    cTitle = item->text(1);
    cUrl = item->text(4);
  }  
   else
    if (cPanel->GetPlaying())
    {
      videoWin->slotStopPlayback();
      cPanel->ResetControlPanel();
    }  
  
}


void KMP::slotPlayNext()
{
  if (hasChapters)
  {
    videoWin->PlayNextChapter();
    return;
  }
    
  if (playSingleUrl) return;
    
  QListViewItem* item = playList->GetNext();

  if (!item) return;
 
   
    if (videoWin->PlayMRL(item->text(4), item->text(1), true))
     {
       playRecent->addURL(item->text(4));
       cTitle = item->text(1);
       cUrl = item->text(4);
     }  
     else 
     {
        videoWin->slotStopPlayback();
        cPanel->ResetControlPanel();
     }
      
}
 


void KMP::slotPlayPrev()
{

  if (hasChapters)
  {
    videoWin->PlayPreviousChapter();
    return;
  }
  
  if (playSingleUrl) return;
  
   QListViewItem* item = playList->GetPrevious();

   if (!item) return;
   
   
    if (videoWin->PlayMRL(item->text(4), item->text(1), true))
     {
      playRecent->addURL(item->text(4));
      cTitle = item->text(1);
      cUrl = item->text(4);
     } 
     else 
     {
       videoWin->slotStopPlayback();
       cPanel->ResetControlPanel();
     }
      
}


void KMP::slotEndOfPlayback()
{
  if (!cPanel->GetPlaying()) return; //end of logo, error

  if (playSingleUrl)
   {
     videoWin->slotStopPlayback();
     cPanel->ResetControlPanel();
     tRow->slotChangeText(i18n("Ready."));
     playSingleUrl = false;
     return;
   }

  
   QListViewItem* item = playList->GetNext();

   if (!item)
   {
     videoWin->slotStopPlayback();
     cPanel->ResetControlPanel();
     tRow->slotChangeText(i18n("Ready."));
   }
   else
 
   {  
     if (videoWin->PlayMRL(item->text(4), item->text(1), true))
      {
       playRecent->addURL(item->text(4));
       cTitle = item->text(1);
       cUrl = item->text(4);
      } 
      else 
      {
        videoWin->slotStopPlayback();
        cPanel->ResetControlPanel();
      }
   }   
}


void KMP::slotStop()
{
  videoWin->slotStopPlayback();
  cPanel->ResetControlPanel();
  playSingleUrl = false;
  tRow->slotChangeText(i18n("Ready."));
}

void KMP::slotPlayRecent(const KURL& url)
{

//  PRINT("Open recent url: " << url.prettyURL())

  QString title;
  
  if (url.isLocalFile())
       title = url.fileName();
     else
       title = url.prettyURL();

  
  if (videoWin->PlayMRL(url.prettyURL(), title, false))
  {
    cPanel->SetPlaying();
    playSingleUrl = true;
    cUrl = url.prettyURL();
    cTitle = title;
  }
  else
    if (cPanel->GetPlaying())
    {
      videoWin->slotStopPlayback();
      cPanel->ResetControlPanel();
    }  
}


void KMP::slotPlayLOGO()
{
  videoWin->PlayLOGO();
}   


//**************************************************** 

void KMP::slotQuit()
{
  
  if (videoWin->xineRunning)
  {
   
    SaveConfig();
    playList->SaveCurrentPlaylist(); //if any
    videoWin->ExitXine();
  }  

  
  delete playList;
  delete configWin;
  delete videoSettings;
  delete osDisplay;
  delete videoWin;
  delete tRow;
  delete cPanel;
  
  LOG("Bye bye.");  
  KApplication::exit(0);
}  

//************************************************


void KMP::slotOpenFile()
{
 
  KURL::List urlList;

   urlList = KFileDialog::getOpenURLs(QDir::homeDirPath(), QString::null, 0, i18n("Open File(s)"));

  
  if (!urlList.count()) return;

  playList->slotClearList();
  playList->Add(urlList, NULL);
  
  slotPlay();
}

void KMP::slotOpenUrl()
{
  KURL kurl = KURLRequesterDlg::getURL(QString::null, 0, i18n("Open URL"));

  if (!kurl.isValid()) return;
  
  playList->slotClearList();
  playList->Add(kurl, NULL);
  slotPlay();
}


void KMP::slotToggleFullscreen()
{
  if (viewFullscreen->isChecked())
  {
    // go fullscreen
    videoWin->LockXineDisplay();
    menuBar->hide();
    tRow->hide();
    cPanel->hide();
    showFullScreen();
    videoWin->UnlockXineDisplay();
    videoWin->StartMouseHideTimer();
  }
  else
  {
    
    videoWin->LockXineDisplay();
    menuBar->show();
    tRow->show();
    cPanel->show();
    showNormal();
    videoWin->UnlockXineDisplay();
    videoWin->StopMouseHideTimer();
  }
}

void KMP::slotPlayAutoplayPlugin(const QString& plugin)
{
  QStringList mrlList;


  if (videoWin->GetAutoplayPluginMrl(plugin, mrlList));
  {
    playList->slotClearList();
    playList->AddAutoplayUrls(mrlList, NULL);
    slotPlay();
  }
}

void KMP::slotHasChapters(bool c)
{
  hasChapters = c;
}  


void KMP::slotShowPlayList()
{
  if (playList->isVisible())
    playList->hide();
   else
    playList->show();
}


void KMP::slotShowSetup()
{
  if (configWin->isVisible())
    configWin->hide();
   else
    configWin->show();
}      


void KMP::slotEndlessMode()
{
  if (playEndlessPlay->isChecked())
    playList->SetEndless(true);
   else
    playList->SetEndless(false);
}


void KMP::slotRandomPlay()
{
  if (playRandomPlay->isChecked())
    playList->SetRandomPlay(true);
   else
    playList->SetRandomPlay(false);
}


void KMP::slotGenConfig(bool loadPl)
{
  loadPlaylistAtStartup = loadPl;
}


void KMP::slotConfigXine()
{
   new XineConfig(videoWin->GetXineEngine());
}


void KMP::slotVideoSettings()
{
  videoSettings->show();
}  



//******* take a screenshot ***********

void KMP::slotScreenshot()
{
  int width, height;
  double scaleFactor;
  uchar* rgb32BitData = NULL;

  videoWin->GetScreenshot(rgb32BitData, width, height, scaleFactor);

  if (rgb32BitData == NULL)
  {
    LOG("can't take screenshot")
    return;
  }

  QImage shot(rgb32BitData, width, height, 32 , (uint*)&RGB_MASK , 0  , QImage::IgnoreEndian);

 // PRINT("Screenshot: " << width << "x" << height << ", scale with " << scaleFactor)
  
  if (scaleFactor >= 1.0)
    width = (int)((double) width * scaleFactor);  
  else
    height = (int) ((double) height / scaleFactor);
    
  shot = shot.smoothScale(width, height);
  
//  PRINT("...to: " << shot.width() << "x" << shot.height())

//***** save


  KFileDialog dlg(QDir::homeDirPath(), i18n("*.png|PNG-File\n*.bmp|BMP-File\n*.xbm|XBM-File"),
                          0, "save screenshot", true);

  dlg.setOperationMode(KFileDialog::Saving);
  dlg.setCaption(i18n("Save screenshot as..."));
  dlg.setSelection("screenshot.png");

  ScreenshotPreview* prev = new ScreenshotPreview(shot, &dlg);
  dlg.setPreviewWidget(prev);

  dlg.exec();

  QString fileName = dlg.selectedFile();

  if (fileName.isEmpty())
  {
    delete [] rgb32BitData;
    return;
  }  

  QString type = dlg.currentFilter();
  type = (type.remove(0,2)).upper();
  PRINT("Save screenshot as " << type)
  
 
  if  (!shot.save(fileName, type))
    ERROR(i18n("Screenshot not saved successfully!"))
    

  delete [] rgb32BitData;
}

//*********************************


void KMP::slotToggleTv()
{
   /* not implemented yet */
}



//******* kbookmarkowner reimplementation


void KMP::openBookmarkURL(const QString& url)
{

//  PRINT("Play bookmark entry: " << url)
  slotPlayRecent(url);
}


QString KMP::currentTitle() const
{
   return cTitle;
}


QString KMP::currentURL() const
{
  return cUrl;
}


                    



