/***************************************************************************
                          videowindow.cpp  -  description
                             -------------------
    begin                : Fre Apr 18 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <klocale.h>
#include <kurl.h>
#include <kstandarddirs.h>

#include  <qdir.h>
#include <qcursor.h>
#include <qimage.h>


#include "videowindow.h"

#include <X11/keysym.h>
#ifdef HAVE_XTEST
#include <X11/extensions/XTest.h>
#endif

// missing stuff from X includes
#ifndef XShmGetEventBase
extern int XShmGetEventBase(Display *);
#endif   


#define MAX(x,y) ( (x) > (y) ? (x) : (y) )


VideoWindow::VideoWindow(QWidget *parent, const char *name ) : QWidget(parent,name)
{
  xineEngine=NULL; xineStream=NULL; audioDriver=NULL; videoDriver=NULL;
  visualPlugin=NULL; eventQueue=NULL; xinePost = NULL;  xineRunning=false;
  xineDisplay = NULL; currentZoom = 100; 

  setPaletteBackgroundColor(QColor(0,0,0));
  setMouseTracking(true);  // get mouse move events


  xineErrors << i18n("No Error");
  xineErrors << i18n("Xine-Error: No Input Plugin");
  xineErrors << i18n("Xine-Error: No Demux Plugin");
  xineErrors << i18n("Xine-Error: Demux failed");
  xineErrors << i18n("Xine-Error: Malformed MRL");
  xineErrors << i18n("Xine-Error: Unknown error");

  playMode = NORMAL_PLAY;

  connect(&posTimer, SIGNAL(timeout()), this, SLOT(slotGetPosition()));
  connect(&lengthInfoTimer, SIGNAL(timeout()), this, SLOT(slotEmitLengthInfo()));
  connect(&screensaverTimer, SIGNAL(timeout()), this, SLOT(slotFakeKeyEvent()));
  connect(&mouseHideTimer, SIGNAL(timeout()), this, SLOT(slotHideMouse()));
}


VideoWindow::~VideoWindow()
{
}


void VideoWindow::LockXineDisplay() const
{
  XLockDisplay(xineDisplay);
}

void VideoWindow::UnlockXineDisplay() const
{
  XUnlockDisplay(xineDisplay);
}    

void VideoWindow::DestSizeCallback(void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
                       int* dest_width, int* dest_height, double* dest_aspect)

{

  if (p==NULL) return;


  VideoWindow* vw = (VideoWindow*) p;

  
  *dest_width=vw->width();
  *dest_height=vw->height();
  *dest_aspect=vw->displayRatio;

}



void VideoWindow::FrameOutputCallback(void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
                          int* dest_x, int* dest_y, int* dest_width, int* dest_height,
                          double* dest_aspect, int* win_x, int* win_y)

{

  if (p==NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  *dest_x=0;
  *dest_y=0;
  *dest_width=vw->width();
  *dest_height=vw->height();
  *win_x= vw->x();
  *win_y= vw->y();
  *dest_aspect=vw->displayRatio;

}

void VideoWindow::XineEventListener(void *p, const xine_event_t* xineEvent)
{

  if (p==NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

 switch (xineEvent->type)
  {
    case XINE_EVENT_UI_PLAYBACK_FINISHED:
       {
       //  PRINT("Playback finished")
         vw->signalPlaybackFinished();
         break; 
       }
    case XINE_EVENT_UI_CHANNELS_CHANGED:
      {
      //  PRINT("New channel info")

        int i,j,channels;
        char* lang = new char[10];
        QString num;
        vw->audioCh.clear();
        vw->subCh.clear();

      //*** get audio channels   
        channels = MAX(6, xine_get_stream_info(vw->xineStream, XINE_STREAM_INFO_MAX_AUDIO_CHANNEL));
        for(i = 0; i < channels; i++)
        {
          if (xine_get_audio_lang(vw->xineStream, i, lang))
            vw->audioCh << lang;
           else
            vw->audioCh << i18n("Ch ") + num.setNum(i+1);
        }     
      
        //*** get subtitle channels
        channels = MAX(6, xine_get_stream_info(vw->xineStream, XINE_STREAM_INFO_MAX_SPU_CHANNEL));
        for(j = 0; j < channels; j++)
        {
          if (xine_get_spu_lang(vw->xineStream, j, lang))
            vw->subCh << lang;
           else
            vw->subCh << i18n("Ch ") + num.setNum(j+1);
        }    

        delete lang;

        int currentAudio = xine_get_param(vw->xineStream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
        int currentSub = xine_get_param(vw->xineStream, XINE_PARAM_SPU_CHANNEL);
 
        if (currentAudio > (i-1))
        {
       //   PRINT("Reset Audio Channel")
          vw->slotSetAudioChannel(1);
          currentAudio = -1;
        } 
          
        if (currentSub > (j-1))
        {
       //   PRINT("Reset Subtitle Channel")
          vw->slotSetSubtitleChannel(0);
          currentSub = -1;
         }

         vw->signalNewChannels(vw->audioCh, vw->subCh, currentAudio, currentSub);
         break;
      }
    case XINE_EVENT_UI_SET_TITLE:
       {
        PRINT("New title");
        xine_ui_data_t* xd=(xine_ui_data_t*)xineEvent->data;
        vw->trackInfoRow = xd->str;
        vw->signalNewInfo(xd->str);
        break;
       }
     case XINE_EVENT_PROGRESS:
       {
      //  PRINT("Progress data")
        xine_progress_data_t* pd=(xine_progress_data_t*)xineEvent->data;

        QString perc;
        perc.setNum(pd->percent);
        perc.append(" %");
        
        QString progress = pd->description;
        progress.append("  ");
        progress.append(perc);
        
        vw->signalNewInfo(progress);

        if (pd->percent == 100)
         vw->signalNewInfo(vw->trackInfoRow);
        
        break;
        }
     case XINE_EVENT_UI_NUM_BUTTONS:
      {
        xine_ui_data_t* buttons=(xine_ui_data_t*)xineEvent->data;

        if (buttons->num_buttons==0)
           vw->setCursor(QCursor(Qt::ArrowCursor));
          else
           vw->setCursor(QCursor(Qt::PointingHandCursor));
        break;
       }
     case XINE_EVENT_MRL_REFERENCE:
       {
        PRINT("New mrl reference")
        xine_mrl_reference_data_t* mrldata=(xine_mrl_reference_data_t*)xineEvent->data;
        vw->signalNewMrlReference(mrldata->mrl); 
        break;
       } 
    }
}


void VideoWindow::VisualizationChangedCallback(void* p, xine_cfg_entry_t* entry)
{
  if (p==NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  PRINT("New visualization plugin: " << entry->enum_values[entry->num_value] << entry->num_value)
  if (entry->num_value>0)
    vw->visualPluginName = entry->enum_values[entry->num_value];
   else
    vw->visualPluginName = QString::null;

  if (!(xine_get_status(vw->xineStream) == XINE_STATUS_PLAY))
   if (vw->xinePost)
   {
     vw->postAudioSource  = xine_get_audio_source (vw->xineStream);
     xine_post_wire_audio_port (vw->postAudioSource, vw->audioDriver);
     xine_post_dispose (vw->xineEngine, vw->xinePost);
     vw->xinePost=NULL;
   }
}


void VideoWindow::LogoPathChangedCallback(void* p, xine_cfg_entry_t* entry)
{
  if (p==NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  vw->logoPath = entry->str_value;
}  


void* VideoWindow::EventLoop(void* p)
{
  if (p == NULL) return NULL;
  VideoWindow* vw = (VideoWindow*) p;

  PRINT("Start event loop...")

  XEvent event;

 while(1)
 {

    XNextEvent (vw->xineDisplay, &event);

    
    if (event.type == Expose)
    {
      // PRINT("Expose event")
      xine_gui_send_vo_data (vw->xineStream, XINE_GUI_SEND_EXPOSE_EVENT, &event);
    }    


    if (event.type == vw->shmEventId)
    {
      xine_gui_send_vo_data (vw->xineStream, XINE_GUI_SEND_COMPLETION_EVENT, &event);
    }


    if (event.type == ClientMessage)
    {
      XClientMessageEvent *cm = (XClientMessageEvent *) &event;

      if (cm->message_type == vw->atomQuit)
       break;
    }   
   

  }

  PRINT("Exiting event loop...")
  pthread_exit(NULL);
  return NULL;
}    
  
  


//********************** INIT


bool VideoWindow::InitXine()
{

 double    res_h, res_v, aspect_diff;


//************* INIT XINE DISPLAY
 
  xineWindow= winId();

    
  if (!XInitThreads ()) {
    ERROR(i18n("XInitThreads failed!"));
    return false;
  }

  xineDisplay = XOpenDisplay(NULL);

  if (!xineDisplay) {
    ERROR(i18n("Failed to connect to X-Server!"));
    return false;
  }

  
  LockXineDisplay();

  xineScreen= DefaultScreen(xineDisplay);

  
  if (XShmQueryExtension (xineDisplay) == True)
  {
    shmEventId = XShmGetEventBase (xineDisplay) + ShmCompletion;
  }
  else
  {
    LOG("No SHM-Extension");
    shmEventId = -1;
  }             

  // create a simple window or comunication only
  xComWindow = XCreateSimpleWindow( xineDisplay, DefaultRootWindow(xineDisplay), 0, 0, 1, 1, 0, 0, 0 );

  XSelectInput (xineDisplay, xineWindow, ExposureMask);
  

// determine display aspect ratio 
  res_h = (DisplayWidth  (xineDisplay, xineScreen)*1000
	  / DisplayWidthMM (xineDisplay, xineScreen));
  res_v = (DisplayHeight (xineDisplay, xineScreen)*1000
	  / DisplayHeightMM (xineDisplay, xineScreen));
  displayRatio = res_v / res_h;
  aspect_diff = displayRatio - 1.0;
  if ((aspect_diff < 0.01) && (aspect_diff > -0.01)) {
    displayRatio   = 1.0;
  }
  PRINT("DisplayAspectRatio: " << displayRatio);

  atomQuit = XInternAtom(xineDisplay, "QUIT_SIGNAL", False );

  UnlockXineDisplay();


//********* INIT XINE ENGINE

  LOG("Using Xine Version " << xine_get_version_string())

  xineEngine = xine_new();
  if (!xineEngine)
  {
    ERROR(i18n("Can't init Xine Engine!"));
    return false;
  }  

// load config

  configFile=QDir::homeDirPath();
  configFile.append("/.kaffeine/config");

  if (!QFile::exists(configFile))
    LOG("No config file found, will create...")
  else
    xine_config_load (xineEngine, configFile);


   PRINT("Post-init xine-engine")
   xine_init(xineEngine);



//************** configuration ******************

  
  char** drivers = NULL;
  char** audioChoices = NULL;
  int i=0;

  drivers = (char **) xine_list_audio_output_plugins (xineEngine);
  audioChoices = new char*[15];
  for(i=0; i<15; i++) audioChoices[i]= new char[10];
  audioChoices[0]=const_cast<char*>("auto");
  i=0;
  while(drivers[i])
  {
    audioChoices[i+1]=drivers[i];
    i++;
  }  
  audioChoices[i+1]=NULL;

  QString* audioInfo = new QString(i18n("Audiodriver to use (default: auto)"));
  i = xine_config_register_enum(xineEngine, "gui.audiodriver", 0,
                   audioChoices, audioInfo->latin1(), NULL, 10, NULL, NULL);
  audioDriverName = audioChoices[i];
  delete [] drivers;

  LOG("Use Audio Driver " << audioDriverName);

  i=0;
  char** videoChoices = NULL;
  drivers = (char **) xine_list_video_output_plugins (xineEngine);
  videoChoices = new char*[15];
  for(i=0; i<15; i++) videoChoices[i]= new char[10];
  i=0;
  videoChoices[0]=const_cast<char*>("auto");
  while(drivers[i])
  {
    videoChoices[i+1]=drivers[i];
    i++;
  }
  videoChoices[i+1]=NULL;

  QString* videoInfo = new QString(i18n("Videodriver to use (default: auto)"));
  i = xine_config_register_enum(xineEngine, "gui.videodriver", 0,
                   videoChoices, videoInfo->latin1(), NULL, 10, NULL, NULL);
  videoDriverName = videoChoices[i];
  delete [] drivers;

  LOG("Use Video Driver " << videoDriverName);


  i=0;
  int goom = 1;
  char** visualChoices = NULL;
  drivers = (char **) xine_list_post_plugins_typed(xineEngine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
  visualChoices = new char*[15];
  for(i=0; i<15; i++) visualChoices[i]= new char[10];
  i=0;
  visualChoices[0]=const_cast<char*>("none");
  while(drivers[i])        
  {
    visualChoices[i+1]=drivers[i];
    if (strcmp(visualChoices[i+1], "goom") == 0) goom = i+1;
    i++;
  }
  visualChoices[i+1]=NULL;

  QString* visualInfo = new QString(i18n("Visualization plugin to use (default: goom)"));
  i = xine_config_register_enum(xineEngine, "gui.visualization", goom,
                   visualChoices, visualInfo->latin1() , NULL, 10,
                   &VideoWindow::VisualizationChangedCallback, this);
  visualPluginName = visualChoices[i];
  delete [] drivers;

  LOG("Use Visualization Plugin " << visualPluginName);



  x11_visual_t visual;

// create video port 
  visual.display          = xineDisplay;
  visual.screen           = xineScreen;
  visual.d                = xineWindow;
  visual.dest_size_cb     = &VideoWindow::DestSizeCallback;
  visual.frame_output_cb  = &VideoWindow::FrameOutputCallback;
  visual.user_data        = (void*)this;

  PRINT("Init video driver")

  videoDriver = xine_open_video_driver(xineEngine,
               videoDriverName,  XINE_VISUAL_TYPE_X11,
               (void *) &(visual));
  if (!videoDriver)
  {
    ERROR(i18n("Can't init Video Driver!"));
    return false;
  }               

  PRINT("Init audio driver")
  
  audioDriver = xine_open_audio_driver (xineEngine, audioDriverName, NULL);
  if (!audioDriver)
  {
    ERROR(i18n("Can't init Audio Driver!"));
    return false;
  }  

  PRINT("Open stream")
  
  xineStream  = xine_stream_new(xineEngine, audioDriver, videoDriver);
  if (!xineStream)
  {
    ERROR(i18n("Can't create a new Xine-Stream!"));
    return false;
  }

//*********** test post plugin

  PRINT("Testing visualization plugin");
  visualPlugin = xine_post_init (xineEngine, visualPluginName, 0,
                         &audioDriver,&videoDriver);
  if (!visualPlugin)
  {
    visualPluginName = QString::null;
    LOG("Can't use Visualiziation Plugin")
  }
  else
   xine_post_dispose(xineEngine, visualPlugin);  
      

//*********** event handling

  eventQueue = xine_event_new_queue (xineStream);
  xine_event_create_listener_thread(eventQueue, &VideoWindow::XineEventListener, (void*)this);
  

//*********** start event thread


   if (pthread_create(&eventThread, NULL, &VideoWindow::EventLoop, this))
   {
     ERROR(i18n("Can't create thread!"))
     return false;
   }  
 

//********* config stuff:


  QStringList dirs = KGlobal::dirs()->findDirs("data","kaffeine");
  logoPath = dirs[0];  // global kde path
  logoPath.append("logo.avi");

   PRINT("Logopath: " << logoPath);

  QString* logoInfo = new QString(i18n("Absolut path to the logo URL (png file for example)"));
  logoPath = xine_config_register_string(xineEngine, "gui.logo_path", logoPath,
                     logoInfo->latin1(), NULL, 10,
                     &VideoWindow::LogoPathChangedCallback, this);


// get a keycode for a faked key event to hold down screensaver

#ifdef HAVE_XTEST
int a,b,c,d;
  haveXTest= XTestQueryExtension(x11Display(), &a, &b, &c, &d);
  if (haveXTest)
    xTestKeycode=XKeysymToKeycode(x11Display(), XK_Shift_L);
  
#endif    

         
                     
//**** success
  
      
  PRINT("Xine Init Success");
  xineRunning=true;
  return true;
}


void VideoWindow::ExitXine()
{

 if ((xine_get_status(xineStream) == XINE_STATUS_PLAY) || (xine_get_status(xineStream) == XINE_STATUS_IDLE))
   xine_close(xineStream);


  XClientMessageEvent cm;
  cm.type = ClientMessage;
  cm.window = xineWindow;
  cm.message_type = atomQuit;
  cm.format = 32;


  XSendEvent(xineDisplay, xComWindow, True, 0, (XEvent*) &cm );  //send quit signal to event loop
  XFlush(xineDisplay);

  PRINT("Wait for event loop to die...")
  pthread_join(eventThread, NULL);

  
  PRINT("Save " << configFile);
  xine_config_save(xineEngine, configFile);

  LOG("Shut down xine engine");
  xine_exit(xineEngine);

  XSync(xineDisplay, False);
  XDestroyWindow(xineDisplay, xComWindow);
  XCloseDisplay(xineDisplay);  //close xine display
  xineDisplay=NULL;
}


bool VideoWindow::PlayMRL(const QString& mrl, const QString& title, bool returnInfo)
{
  
  if (xine_get_status(xineStream) == XINE_STATUS_PLAY)
    xine_stop(xineStream);

 
  emit signalNewInfo(i18n("Opening..."));

  setCursor(QCursor(Qt::WaitCursor));
  
  if (!xine_open(xineStream, mrl))
  {

    int errCode = xine_get_error(xineStream);
    if ( (errCode < 0) || (errCode > 5) ) errCode = 6;
    
    QString err = xineErrors[errCode];
    emit signalNewInfo(err);

    if (returnInfo)
    {
      emit signalStreamInfo(err);
      emit signalLengthInfo(i18n("*Error*"));
    }  
    setCursor(QCursor(Qt::ArrowCursor));
    return false;
  }  

  //******* use visualization ?

 if (!xine_get_stream_info (xineStream, XINE_STREAM_INFO_HAS_VIDEO))
 {

   if (visualPluginName && (!xinePost))
   {
    xinePost = xine_post_init (xineEngine, visualPluginName, 0,
          &audioDriver,
          &videoDriver);

     postAudioSource = xine_get_audio_source (xineStream);
     postInput = (xine_post_in_t*)xine_post_input (xinePost, const_cast<char*>("audio in"));
     xine_post_wire (postAudioSource, postInput);
   }
  }
  else
  {
    if (xinePost)
    {
      postAudioSource  = xine_get_audio_source (xineStream);
      xine_post_wire_audio_port (postAudioSource, audioDriver);
      xine_post_dispose (xineEngine, xinePost);
      xinePost=NULL;
    }
   }

//********** play   
  
  if (!xine_play(xineStream, 0,0))
  {
    int errCode = xine_get_error(xineStream);
    if ( (errCode < 0) || (errCode > 5) ) errCode = 6;
    
    QString err = xineErrors[errCode];
    emit signalNewInfo(err);
    if (returnInfo)
    {
      emit signalStreamInfo(err);
      emit signalLengthInfo(i18n("*Error*"));
    }  
    setCursor(QCursor(Qt::ArrowCursor));
    return false;
  }


  

// have the stream chapters ?
  if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_CHAPTERS))
    emit signalHasChapters(true);
   else
    emit signalHasChapters(false);
 
//******** information **********

  QString trackTitle = title;

  QString tmp;;
  QString num;
  QString metaInfo;
  
  tmp = xine_get_meta_info(xineStream, XINE_META_INFO_TITLE);
  if (tmp)  //meta info?
  {
    QString metaTitle =  QString(tmp);
    QString metaArtist= QString(xine_get_meta_info(xineStream, XINE_META_INFO_ARTIST));
    QString metaAlbum =  QString(xine_get_meta_info(xineStream, XINE_META_INFO_ALBUM));
    metaInfo = QString(metaString).arg(metaArtist).arg(metaTitle).arg(metaAlbum);
    trackTitle = metaInfo;
    if ((metaInfo != title) && (returnInfo))
      emit signalMetaInfo(metaInfo);
  }
  
   trackInfoRow = trackTitle;  
       
  
   trackInfoRow.append("  ***");

   QString streamInfo;
   
   if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_VIDEO))
   {
     streamInfo.append(xine_get_meta_info(xineStream, XINE_META_INFO_VIDEOCODEC));
     streamInfo.append(" (");
     streamInfo.append(num.setNum(xine_get_stream_info(xineStream, XINE_STREAM_INFO_VIDEO_WIDTH)));
     streamInfo.append("x");
     streamInfo.append(num.setNum(xine_get_stream_info(xineStream, XINE_STREAM_INFO_VIDEO_HEIGHT)));
     streamInfo.append(")");
   }

   if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_AUDIO))
   {
     streamInfo.append(" ");
     streamInfo.append(xine_get_meta_info(xineStream, XINE_META_INFO_AUDIOCODEC));
     streamInfo.append(" (");
     streamInfo.append(num.setNum(xine_get_stream_info(xineStream, XINE_STREAM_INFO_AUDIO_BITRATE)/1000));
     streamInfo.append("kb)");
   }

   trackInfoRow.append(streamInfo);    
                     
//************  

  emit signalNewInfo(trackInfoRow);

  if (returnInfo)
  {
    streamInfo = QString("%1: %2").arg(xine_get_meta_info(xineStream, XINE_META_INFO_INPUT_PLUGIN)).arg(streamInfo);
    emit signalStreamInfo(streamInfo);
  }  
  
  slotSetAudioChannel(1); //refresh channel info  
  
  posTimer.start(500);

 //wait for available track length info
  if (returnInfo)
  {
    lengthInfoTimer.start(1000, true); //single shot
  }  
    
  setCursor(QCursor(Qt::ArrowCursor));
                
  emit signalShowOSD(trackTitle);

  return true;
}


void VideoWindow::GetAutoplayPlugins(QStringList& autoPlayList) const
{
  char** pluginIds = NULL;
  int i=0;

  pluginIds = (char**)xine_get_autoplay_input_plugin_ids(xineEngine);

  while(pluginIds[i])
  {
    autoPlayList << pluginIds[i];
   
    
    autoPlayList << xine_get_input_plugin_description(xineEngine, pluginIds[i]);    
    i++;
  }  
}

 
bool VideoWindow::GetAutoplayPluginMrl(const QString& plugin, QStringList& list)
{
  char** mrls=NULL;
  int num;
  int i=0;

  mrls = xine_get_autoplay_mrls(xineEngine, plugin, &num);
  if (mrls)
   {
    while (mrls[i])
      { list << mrls[i];
        i++;
      }  
    
    return true;
   } 
   else 
   {
     QString error(i18n("Error: No "));
     error.append(plugin);
     error.append(i18n(", or wrong path to device."));
     signalNewInfo(error);   
    return false;
   } 
}


int VideoWindow::GetVolume() const
{
  return xine_get_param(xineStream, XINE_PARAM_AUDIO_VOLUME);
}


void VideoWindow::slotSetVolume(int vol)
{
  xine_set_param(xineStream, XINE_PARAM_AUDIO_VOLUME, -vol);
}

void VideoWindow::mouseMoveEvent(QMouseEvent* mev)
{
   if (cursor().shape() == Qt::BlankCursor)
     setCursor(QCursor(Qt::ArrowCursor));

    x11_rectangle_t   rect;
    xine_event_t      event;
    xine_input_data_t input;

    rect.x = mev->x();
    rect.y = mev->y();
    rect.w = 0;
    rect.h = 0;

    xine_gui_send_vo_data (xineStream,
               XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO,
                (void*)&rect);

    event.type        = XINE_EVENT_INPUT_MOUSE_MOVE;
    event.data        = &input;
    event.data_length = sizeof(input);
    input.button      = 0; 
    input.x           = rect.x;
    input.y           = rect.y;
    xine_event_send (xineStream, &event);
}

void VideoWindow::mousePressEvent(QMouseEvent* mev)
{

   if (mev->button() == Qt::LeftButton)
   {
    x11_rectangle_t   rect;
    xine_event_t      event;
    xine_input_data_t input;

    rect.x = mev->x();
    rect.y = mev->y();
    rect.w = 0;
    rect.h = 0;

    xine_gui_send_vo_data (xineStream,
                XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO,
                   (void*)&rect);

    event.type        = XINE_EVENT_INPUT_MOUSE_BUTTON;
    event.data        = &input;
    event.data_length = sizeof(input);
    input.button      = 1;
    input.x           = rect.x;
    input.y           = rect.y;
    xine_event_send (xineStream, &event);
   }
}



void VideoWindow::ReDraw() const
{
  if (!xineDisplay) return;

  // create expose event
  
  XExposeEvent eev;
  eev.type = Expose;
  eev.display = xineDisplay;
  eev.window = xineWindow;
  eev.x = x();
  eev.y = y();
  eev.width = width();
  eev.height = height();

  XSendEvent(xineDisplay, xComWindow, True, 0, (XEvent*) &eev);
  XFlush(xineDisplay);  //process immediately

} 


void VideoWindow::PlayNextChapter() const
{

  xine_event_t xev;

  xev.type = XINE_EVENT_INPUT_NEXT;
  xev.data = NULL;
  xev.data_length = 0;
  
  xine_event_send(xineStream, &xev);
}  

void VideoWindow::PlayPreviousChapter() const
{

  xine_event_t xev;

  xev.type = XINE_EVENT_INPUT_PREVIOUS;
  xev.data = NULL;
  xev.data_length = 0;

  xine_event_send(xineStream, &xev);
}  


void VideoWindow::slotStopPlayback()
{
    posTimer.stop();
    PlayLOGO();
}


void VideoWindow::PlayLOGO() 
{
  if (xine_get_status(xineStream) == XINE_STATUS_PLAY)
    xine_stop(xineStream);

  if (!logoPath.isNull())
   {
   
     if(!xine_open(xineStream, logoPath))
     {
       LOG("Invalid logo file")
       logoPath = QString::null;
       return;
     }  

     xine_play(xineStream, 0,0);
    
   }  
}



void VideoWindow::slotSpeedPause()
{
  xine_set_param(xineStream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
  posTimer.stop();
  signalNewInfo("Pause.");
}


void VideoWindow::slotSpeedNormal()
{
  xine_set_param(xineStream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
  posTimer.start(500);
  signalNewInfo(trackInfoRow);
}

QString VideoWindow::GetMimeTypes() const
{
  
  return xine_get_mime_types(xineEngine);
}

void VideoWindow::slotSetAudioChannel(int ch)
{
  xine_set_param(xineStream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, ch-2);
}  


void VideoWindow::slotSetSubtitleChannel(int ch)
{
  xine_set_param(xineStream, XINE_PARAM_SPU_CHANNEL, ch-1);
}

void VideoWindow::slotTogglePlayMode()
{
  switch (playMode)
 {
   case NORMAL_PLAY:
   {
     playMode = REPEAT_PLAY;
     break;
   }
   case REPEAT_PLAY:
   {
     playMode = PERCENT_PLAY;
     break;
   }
   case PERCENT_PLAY:
   {
     playMode = NORMAL_PLAY;
     break;
   }
 }
}

QString VideoWindow::GetTimeString(int msec)
{
  int hours;
  int min;
  int sec;
  QString tmp;
  QString t;

    msec = msec/1000;  //sec
    hours = msec/3600;
    t = t.setNum(hours);
    t.append(":");

    msec = msec - (hours*3600);
    min = msec / 60;
    tmp = tmp.setNum(min);
    tmp = tmp.rightJustify(2, '0');
    t.append(tmp);
    t.append(":");

    sec = msec - (min*60);
    tmp = tmp.setNum(sec);
    tmp = tmp.rightJustify(2, '0');
    t.append(tmp);

   return t;                                                                       
}


void VideoWindow::slotGetPosition()
{
  int pos;
  int time;
  int length;
  
  double perc;

  QString t;
  
 
  if (xine_get_pos_length(xineStream, &pos, &time, &length))
  {

    if (!xine_get_stream_info(xineStream, XINE_STREAM_INFO_SEEKABLE)) pos = 0;
  
    switch (playMode)
   { 
   case PERCENT_PLAY:
   {
     perc = pos/655,35;
     time = (int) (perc);
     if ((time<0) || (time>100))
     {
       emit signalNewPosition(pos, "  ??  ");
       return;
     }  
     t = t.setNum(time);
     t = QString(" %1%2 ").arg(t).arg("%");
     emit signalNewPosition(pos, t);
     break;
   }  
   case REPEAT_PLAY:
   {
     time = length - time;
     if (time < 0)
     {
       emit signalNewPosition(pos, " ??:?? ");
       return;
     }  
   }
   case NORMAL_PLAY:
   {
    emit signalNewPosition(pos, GetTimeString(time));
    break;
   }
  } 
 }
 else
  PRINT("No stream-position info")

}


void VideoWindow::slotChangePosition(int pos)
{
  if ( (xine_get_status(xineStream) == XINE_STATUS_PLAY) && (xine_get_stream_info(xineStream, XINE_STREAM_INFO_SEEKABLE)) )
  {
    xine_play(xineStream, pos, 0);
  }
}


void VideoWindow::slotEject()
{
  xine_eject(xineStream);
}


void VideoWindow::slotToggleDeinterlace()
{
  if (xine_get_param(xineStream, XINE_PARAM_VO_DEINTERLACE))
    {
     xine_set_param(xineStream, XINE_PARAM_VO_DEINTERLACE, false);
     PRINT("Turn deinterlace off")
   }  
   else
    xine_set_param(xineStream, XINE_PARAM_VO_DEINTERLACE, true);
}


void VideoWindow::slotAspectRatioAuto()
{
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_AUTO);
}  


void VideoWindow::slotAspectRatio4_3()
{
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_4_3);
}


void VideoWindow::slotAspectRatio16_9()
{
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_ANAMORPHIC);
}


void VideoWindow::slotAspectRatioSquare()
{
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_SQUARE);
}


void VideoWindow::slotZoomOut()
{
  if ((currentZoom - 5) >= 100)
  {
    currentZoom -= 5;
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, currentZoom);
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, currentZoom);
  }
}


void VideoWindow::slotZoomIn()
{
  if ((currentZoom + 5) <= XINE_VO_ZOOM_MAX)
  {
    currentZoom += 5;
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, currentZoom);
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, currentZoom);
  }
}


void VideoWindow::slotZoomOff()
{
  xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, 100);
  xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, 100);
  currentZoom = 100;
}


void VideoWindow::slotEmitLengthInfo()
{
  int pos, time, length;

  if (xine_get_pos_length(xineStream, &pos, &time, &length))
  {
    if (length > 0)
     emit signalLengthInfo(GetTimeString(length));
  }   
  else
   emit signalLengthInfo("   ??");  
}


void VideoWindow::slotSetConfig(const QString& mstr, int ssTimeout)
{
  metaString = mstr;
  screensaverTimeout = ssTimeout;

#ifdef HAVE_XTEST
  if (screensaverTimeout)
    screensaverTimer.start(screensaverTimeout* 60000); //min
   else
    screensaverTimer.stop();
#endif    
}


QString VideoWindow::GetMetaString() const
{
  return this->metaString;
}

int VideoWindow::GetScreensaverTimeout() const
{
  return screensaverTimeout;
}    


void VideoWindow::slotFakeKeyEvent()
{
#ifdef HAVE_XTEST

 PRINT("Fake key event...")

 if (haveXTest)
 {
   XTestFakeKeyEvent(x11Display(), xTestKeycode, true, CurrentTime);
   XTestFakeKeyEvent(x11Display(), xTestKeycode, false, CurrentTime);
   XSync(x11Display(), false);
 }  

#endif 

}


const xine_t* const VideoWindow::GetXineEngine()const
{
  return xineEngine;
}
  

//************* video settings ****************

void VideoWindow::GetVideoSettings(int& hue, int& sat, int& contrast, int& bright,
                                   int& audioAmp, int& avOffset, int& spuOffset) const
{
  hue = xine_get_param(xineStream, XINE_PARAM_VO_HUE);
  sat = xine_get_param(xineStream, XINE_PARAM_VO_SATURATION);
  contrast = xine_get_param(xineStream, XINE_PARAM_VO_CONTRAST);
  bright = xine_get_param(xineStream, XINE_PARAM_VO_BRIGHTNESS);

  audioAmp = xine_get_param(xineStream, XINE_PARAM_AUDIO_AMP_LEVEL);
  avOffset = xine_get_param(xineStream, XINE_PARAM_AV_OFFSET);
  spuOffset = xine_get_param(xineStream, XINE_PARAM_SPU_OFFSET);
}


void VideoWindow::slotSetHue(int hue)
{
  xine_set_param(xineStream, XINE_PARAM_VO_HUE, hue);
}
  

void VideoWindow::slotSetSaturation(int sat)
{
  xine_set_param(xineStream, XINE_PARAM_VO_SATURATION, sat);
}
  

void VideoWindow::slotSetContrast(int contrast)
{
  xine_set_param(xineStream, XINE_PARAM_VO_CONTRAST, contrast);
}
  

void VideoWindow::slotSetBrightness(int bright)
{
  xine_set_param(xineStream, XINE_PARAM_VO_BRIGHTNESS, bright);
}
  

void VideoWindow::slotSetAudioAmp(int amp)
{
  xine_set_param(xineStream, XINE_PARAM_AUDIO_AMP_LEVEL, amp);
}
  

void VideoWindow::slotSetAVOffset(int av)
{
  xine_set_param(xineStream, XINE_PARAM_AV_OFFSET, av);
}
  

void VideoWindow::slotSetSpuOffset(int spu)
{
  xine_set_param(xineStream, XINE_PARAM_SPU_OFFSET, spu);
}


//******** mouse hiding at fullscreen


void VideoWindow::StartMouseHideTimer()
{
  mouseHideTimer.start(4000);
}
  

void VideoWindow::StopMouseHideTimer()
{
  mouseHideTimer.stop();
}
  

void VideoWindow::slotHideMouse()
{
  if (cursor().shape() == Qt::ArrowCursor)
   setCursor(QCursor(Qt::BlankCursor));
}   
  

/************************************************************
 *   Get Screenshot                                         *
 *   code take from gxine !  (thanks)                       *
 *   modified for 32bit output                              *
 ************************************************************/

void yuy2Toyv12 (uint8_t *y, uint8_t *u, uint8_t *v, uint8_t *input,
        int width, int height)
{

  int    i, j, w2;

  w2 = width / 2;

  for (i = 0; i < height; i += 2) {
    for (j = 0; j < w2; j++) {
      /*
       * packed YUV 422 is: Y[i] U[i] Y[i+1] V[i]
       */
      *(y++) = *(input++);
      *(u++) = *(input++);
      *(y++) = *(input++);
      *(v++) = *(input++);
    }

    /*
     * down sampling
     */

    for (j = 0; j < w2; j++) {
      /*
       * skip every second line for U and V
       */
      *(y++) = *(input++);
      input++;
      *(y++) = *(input++);
      input++;
    }
  }
}

/*
 *   Create rgb data from yv12
 */

#define clip_8_bit(val)              \
{                                    \
   if (val < 0)                      \
      val = 0;                       \
   else                              \
      if (val > 255) val = 255;      \
}

uchar * yv12ToRgb (uint8_t *src_y, uint8_t *src_u, uint8_t *src_v,
           int width, int height)
{

  int     i, j;

  int     y, u, v;
  int     r, g, b;

  int     sub_i_uv;
  int     sub_j_uv;

  int     uv_width, uv_height;

  uchar *rgb;

  uv_width  = width / 2;
  uv_height = height / 2;

  rgb = new uchar[(width * height * 4)]; //qt needs a 32bit align
  if (!rgb)
  {
    ERROR(i18n("Not enough memory to make screenshot!"))
    return NULL;
  }  

  for (i = 0; i < height; ++i) {
    /*
     * calculate u & v rows
     */
    sub_i_uv = ((i * uv_height) / height);

    for (j = 0; j < width; ++j) {
      /*
       * calculate u & v columns
       */
      sub_j_uv = ((j * uv_width) / width);

      /***************************************************
       *
       *  Colour conversion from
       *    http://www.inforamp.net/~poynton/notes/colour_and_gamma/ColorFAQ.html#RTFToC30
       *
       *  Thanks to Billy Biggs <vektor@dumbterm.net>
       *  for the pointer and the following conversion.
       *
       *   R' = [ 1.1644         0    1.5960 ]   ([ Y' ]   [  16 ])
       *   G' = [ 1.1644   -0.3918   -0.8130 ] * ([ Cb ] - [ 128 ])
       *   B' = [ 1.1644    2.0172         0 ]   ([ Cr ]   [ 128 ])
       *
       *  Where in xine the above values are represented as
       *
       *   Y' == image->y
       *   Cb == image->u
       *   Cr == image->v
       *
       ***************************************************/

      y = src_y[(i * width) + j] - 16;
      u = src_u[(sub_i_uv * uv_width) + sub_j_uv] - 128;
      v = src_v[(sub_i_uv * uv_width) + sub_j_uv] - 128;

      r = (int)((1.1644 * (double)y) + (1.5960 * (double)v));
      g = (int)((1.1644 * (double)y) - (0.3918 * (double)u) - (0.8130 * (double)v));
      b = (int)((1.1644 * (double)y) + (2.0172 * (double)u));

      clip_8_bit (r);
      clip_8_bit (g);
      clip_8_bit (b);

      rgb[(i * width + j) * 4 + 0] = b; 
      rgb[(i * width + j) * 4 + 1] = g;
      rgb[(i * width + j) * 4 + 2] = r;
      rgb[(i * width + j) * 4 + 3] = 0;
      
    }
  }

  return rgb;
}


void VideoWindow::GetScreenshot(uchar*& rgb32BitData, int& videoWidth, int& videoHeight, double& scaleFactor) const
{

  uint8_t   *yuv = NULL, *y = NULL, *u = NULL, *v =NULL;
  
  int        width, height, ratio, format;
  double     desired_ratio, image_ratio;

  if (!xine_get_current_frame (xineStream, &width, &height, &ratio, &format, NULL))
    return;

  yuv = new uint8_t[((width+8) * (height+1) * 2)];
  if (yuv == NULL)
    {
      ERROR(i18n("Not enough memory to make screenshot!"))
      return;
    }  

  xine_get_current_frame (xineStream, &width, &height, &ratio, &format, yuv);

  videoWidth = width;
  videoHeight = height;

/*
 * convert to yv12 if necessary
 */

  switch (format) {
  case XINE_IMGFMT_YUY2:
    {
      uint8_t *yuy2 = yuv;

      yuv = new uint8_t[(width * height * 2)];
      if (yuv == NULL)
      {
        ERROR(i18n("Not enough memory to make screenshot!"))
        return;
      }  
      y = yuv;
      u = yuv + width * height;
      v = yuv + width * height * 5 / 4;

      yuy2Toyv12 (y, u, v, yuy2, width, height);

      delete [] yuy2;
    }
    break;
  case XINE_IMGFMT_YV12:
    y = yuv;
    u = yuv + width * height;
    v = yuv + width * height * 5 / 4;

    break;
  default:
    {
      LOG("Screenshot: Format " << (char *) &format << "not supported!");
      delete [] yuv;
      return;
    }  
  }

  /*
   * convert to rgb
   */

  rgb32BitData = yv12ToRgb (y, u, v, width, height);


  image_ratio = (double) width / (double) height;


  switch (ratio) {
  case XINE_VO_ASPECT_ANAMORPHIC:  /* anamorphic  */
  case XINE_VO_ASPECT_PAN_SCAN:    /* we display pan&scan as widescreen */
    desired_ratio = 16.0 /9.0;
    break;
  case XINE_VO_ASPECT_DVB:         /* 2.11:1 */
    desired_ratio = 2.11/1.0;
    break;
  case XINE_VO_ASPECT_SQUARE:      /* square pels */
  case XINE_VO_ASPECT_DONT_TOUCH:  /* probably non-mpeg stream => don't touch aspect ratio */
    desired_ratio = image_ratio;
    break;
  default:
    LOG("Unknown/vorbidden aspect ratio " << ratio << " in stream => using 4:3");
  case XINE_VO_ASPECT_4_3:         /* 4:3   */
    desired_ratio = 4.0 / 3.0;
    break;
  }

  scaleFactor = desired_ratio / image_ratio;

  delete [] yuv;
}

  

//******************************************











